#ifndef _MYSQL_SQL_STATEMENT_DECOMPOSER_H_
#define _MYSQL_SQL_STATEMENT_DECOMPOSER_H_


#include "mysql_sql_parser_base.h"
#include "grtsqlparser/sql_statement_decomposer.h"


/** Implements DBMS specifics.
 * 
 * @ingroup sqlparser
 */
class MYSQL_SQL_PARSER_PUBLIC_FUNC Mysql_sql_statement_decomposer
  : protected Mysql_sql_parser_base, public Sql_statement_decomposer
{
public:
  Mysql_sql_statement_decomposer(grt::GRT *grt);

  int decompose_query(const std::string &sql, SelectStatement::Ref select_statement);
  int decompose_view(const std::string &ddl, SelectStatement::Ref select_statement);
  int decompose_view(db_ViewRef view, SelectStatement::Ref select_statement);

protected:
  typedef sigc::slot<Parse_result, const MyxSQLTreeItem *> ProcessSqlStatement;
  int process_sql_statement(const std::string &sql, SelectStatement::Ref select_statement, ProcessSqlStatement do_process_sql_statement_cb);
  int process_sql_statement(const std::string &sql, SelectStatement::Ref select_statement, Mysql_sql_parser_fe &sql_parser_fe);
  int do_process_sql_statement(const MyxSQLTreeItem *tree);
  ProcessSqlStatement _do_process_sql_statement;

protected:
  Parse_result decompose_query(const MyxSQLTreeItem *select_init);
  Parse_result do_decompose_query(const MyxSQLTreeItem *tree);
  SelectStatement::Ref _select_statement;
protected:
  Parse_result do_decompose_view(const MyxSQLTreeItem *tree);
  void expand_wildcards(SelectStatement::Ref select_statement, db_SchemaRef &db_schema, grt::ListRef<db_Schema> &db_schemata);
  std::list<std::string> _view_columns_names;

protected:
  class Null_state_keeper : Mysql_sql_parser_base::Null_state_keeper
  {
  public:
    Null_state_keeper(Mysql_sql_statement_decomposer *sql_parser)
      : Mysql_sql_parser_base::Null_state_keeper(sql_parser), _sql_parser(sql_parser) {}
    ~Null_state_keeper();
  private:
    Mysql_sql_statement_decomposer *_sql_parser;
  };
};


#endif // _MYSQL_SQL_STATEMENT_DECOMPOSER_H_
